# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"ToTensor": "000_utils.ipynb",
         "ToArray": "000_utils.ipynb",
         "To3DTensor": "000_utils.ipynb",
         "To2DTensor": "000_utils.ipynb",
         "To1DTensor": "000_utils.ipynb",
         "To3DArray": "000_utils.ipynb",
         "To2DArray": "000_utils.ipynb",
         "To1DArray": "000_utils.ipynb",
         "To3D": "000_utils.ipynb",
         "To2D": "000_utils.ipynb",
         "To1D": "000_utils.ipynb",
         "To2DPlus": "000_utils.ipynb",
         "To3DPlus": "000_utils.ipynb",
         "To2DPlusTensor": "000_utils.ipynb",
         "To2DPlusArray": "000_utils.ipynb",
         "To3DPlusTensor": "000_utils.ipynb",
         "To3DPlusArray": "000_utils.ipynb",
         "Todtype": "000_utils.ipynb",
         "bytes2size": "000_utils.ipynb",
         "bytes2GB": "000_utils.ipynb",
         "delete_all_in_dir": "000_utils.ipynb",
         "reverse_dict": "000_utils.ipynb",
         "is_tuple": "000_utils.ipynb",
         "itemify": "000_utils.ipynb",
         "is_none": "000_utils.ipynb",
         "ifisnone": "000_utils.ipynb",
         "ifnoneelse": "000_utils.ipynb",
         "ifisnoneelse": "000_utils.ipynb",
         "ifelse": "000_utils.ipynb",
         "stack": "000_utils.ipynb",
         "cycle_dl": "000_utils.ipynb",
         "device": "000_utils.ipynb",
         "defaults.device": "000_utils.ipynb",
         "cpus": "000_utils.ipynb",
         "NumpyTensor": "001_core.ipynb",
         "ToNumpyTensor": "001_core.ipynb",
         "TSTensor": "001_core.ipynb",
         "ToTSTensor": "001_core.ipynb",
         "NumpyTensorBlock": "001_core.ipynb",
         "TSTensorBlock": "001_core.ipynb",
         "Dataset": "001_core.ipynb",
         "NumpyDatasets": "001_core.ipynb",
         "TSDatasets": "001_core.ipynb",
         "add_ds": "001_core.ipynb",
         "NumpyDatasets.add_test": "001_core.ipynb",
         "NumpyDatasets.add_unlabeled": "001_core.ipynb",
         "NumpyDataLoader": "001_core.ipynb",
         "show_tuple": "001_core.ipynb",
         "TSDataLoader": "001_core.ipynb",
         "NumpyDataLoaders": "001_core.ipynb",
         "TSDataLoaders": "001_core.ipynb",
         "save_all": "001_core.ipynb",
         "load_learner_all": "001_core.ipynb",
         "Learner.save_all": "001_core.ipynb",
         "decompress_from_url": "002_data.ipynb",
         "get_UCR_univariate_list": "002_data.ipynb",
         "get_UCR_multivariate_list": "002_data.ipynb",
         "stack_padding": "002_data.ipynb",
         "get_UCR_data": "002_data.ipynb",
         "np_convert": "002_data.ipynb",
         "np_collate": "002_data.ipynb",
         "np_str_obj_array_pattern": "002_data.ipynb",
         "np_collate_err_msg_format": "002_data.ipynb",
         "noop": "102_InceptionTime.ipynb",
         "same_padding1d": "100_layers.ipynb",
         "ZeroPad1d": "100_layers.ipynb",
         "ConvSP1d": "100_layers.ipynb",
         "convlayer": "100_layers.ipynb",
         "ResBlock": "101_ResNet.ipynb",
         "ResNet": "101_ResNet.ipynb",
         "shortcut": "102_InceptionTime.ipynb",
         "Inception": "102_InceptionTime.ipynb",
         "InceptionBlock": "102_InceptionTime.ipynb",
         "InceptionTime": "102_InceptionTime.ipynb"}

modules = ["utils.py",
           "core.py",
           "data.py",
           "models/layers.py",
           "models/ResNet.py",
           "models/InceptionTime.py"]

doc_url = "https://timeseriesai.github.io/timeseriesAI//"

git_url = "https://github.com/timeseriesAI/timeseriesAI/tree/master/"

def custom_doc_links(name): return None



# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/000_utils.ipynb (unless otherwise specified).

__all__ = ['ToTensor', 'ToArray', 'To3DTensor', 'To2DTensor', 'To1DTensor', 'To3DArray', 'To2DArray', 'To1DArray',
           'To3D', 'To2D', 'To1D', 'To2DPlus', 'To3DPlus', 'To2DPlusTensor', 'To2DPlusArray', 'To3DPlusTensor',
           'To3DPlusArray', 'Todtype', 'bytes2size', 'bytes2GB', 'delete_all_in_dir', 'reverse_dict', 'is_tuple',
           'itemify', 'is_none', 'ifisnone', 'ifnoneelse', 'ifisnoneelse', 'ifelse', 'stack', 'cycle_dl', 'device',
           'cpus']

# Cell
from fastai2.imports import *
import torch

# Cell
def ToTensor(o):
    if isinstance(o, torch.Tensor): return o
    elif isinstance(o, np.ndarray):  return torch.from_numpy(o)
    assert False, f"Can't convert {type(o)} to torch.Tensor"


def ToArray(o):
    if isinstance(o, np.ndarray): return o
    elif isinstance(o, torch.Tensor): return o.cpu().numpy()
    assert False, f"Can't convert {type(o)} to np.array"


def To3DTensor(o):
    o = ToTensor(o)
    if o.ndim == 3: return o
    elif o.ndim == 1: return o[None, None]
    elif o.ndim == 2: return o[:, None]
    assert False, f'Please, review input dimensions {o.ndim}'


def To2DTensor(o):
    o = ToTensor(o)
    if o.ndim == 2: return o
    elif o.ndim == 1: return o[None]
    elif o.ndim == 3: return o[0]#torch.squeeze(o, 0)
    assert False, f'Please, review input dimensions {o.ndim}'


def To1DTensor(o):
    o = ToTensor(o)
    if o.ndim == 1: return o
    elif o.ndim == 3: return o[0,0]#torch.squeeze(o, 1)
    if o.ndim == 2: return o[0]#torch.squeeze(o, 0)
    assert False, f'Please, review input dimensions {o.ndim}'


def To3DArray(o):
    o = ToArray(o)
    if o.ndim == 3: return o
    elif o.ndim == 1: return o[None, None]
    elif o.ndim == 2: return o[:, None]
    assert False, f'Please, review input dimensions {o.ndim}'


def To2DArray(o):
    o = ToArray(o)
    if o.ndim == 2: return o
    elif o.ndim == 1: return o[None]
    elif o.ndim == 3: return 0[0]#np.squeeze(o, 0)
    assert False, f'Please, review input dimensions {o.ndim}'


def To1DArray(o):
    o = ToArray(o)
    if o.ndim == 1: return o
    elif o.ndim == 3: o = 0[0,0]#np.squeeze(o, 1)
    elif o.ndim == 2: o = 0[0]#np.squeeze(o, 0)
    assert False, f'Please, review input dimensions {o.ndim}'


def To3D(o):
    if o.ndim == 3: return o
    if isinstance(o, np.ndarray): return To3DArray(o)
    if isinstance(o, torch.Tensor): return To3DTensor(o)


def To2D(o):
    if o.ndim == 2: return o
    if isinstance(o, np.ndarray): return To2DArray(o)
    if isinstance(o, torch.Tensor): return To2DTensor(o)


def To1D(o):
    if o.ndim == 1: return o
    if isinstance(o, np.ndarray): return To1DArray(o)
    if isinstance(o, torch.Tensor): return To1DTensor(o)


def To2DPlus(o):
    if o.ndim >= 2: return o
    if isinstance(o, np.ndarray): return To2DArray(o)
    elif isinstance(o, torch.Tensor): return To2DTensor(o)


def To3DPlus(o):
    if o.ndim >= 3: return o
    if isinstance(o, np.ndarray): return To3DArray(o)
    elif isinstance(o, torch.Tensor): return To3DTensor(o)


def To2DPlusTensor(o):
    return To2DPlus(ToTensor(o))


def To2DPlusArray(o):
    return To2DPlus(ToArray(o))


def To3DPlusTensor(o):
    return To3DPlus(ToTensor(o))


def To3DPlusArray(o):
    return To3DPlus(ToArray(o))


def Todtype(dtype):
    def _to_type(o, dtype=dtype):
        if o.dtype == dtype: return o
        elif isinstance(o, torch.Tensor): o = o.to(dtype=dtype)
        elif isinstance(o, np.ndarray): o = o.astype(dtype)
        return o
    return _to_type

# Cell
def bytes2size(size_bytes):
    if size_bytes == 0: return "0B"
    size_name = ("B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB")
    i = int(math.floor(math.log(size_bytes, 1024)))
    p = math.pow(1024, i)
    s = round(size_bytes / p, 2)
    return "%s %s" % (s, size_name[i])

def bytes2GB(byts):
    return round(byts / math.pow(1024, 3), 2)

# Cell
def delete_all_in_dir(tgt_dir, exception=None):
    if exception is not None and len(L(exception)) > 1: exception = tuple(exception)
    for file in os.listdir(tgt_dir):
        if exception is not None and file.endswith(exception): continue
        file_path = os.path.join(tgt_dir, file)
        if os.path.isfile(file_path) or os.path.islink(file_path): os.unlink(file_path)
        elif os.path.isdir(file_path): shutil.rmtree(file_path)

# Cell
def reverse_dict(dictionary):
    return {v: k for k, v in dictionary.items()}

# Cell
def is_tuple(o): return isinstance(o, tuple)

# Cell
def itemify(*o, tup_id=None):
    items = L(*o).zip()
    if tup_id is not None: return L([item[tup_id] for item in items])
    else: return items

# Cell
def is_none(o):
    return o in [[], [None], None]

def ifisnone(a, b):
    "`a` if `a` is None else `b`"
    return None if is_none(a) else b

def ifnoneelse(a, b, c=None):
    "`b` if `a` is None else `c`"
    return b if a is None else ifnone(c, a)

def ifisnoneelse(a, b, c=None):
    "`b` if `a` is None else `c`"
    return b if is_none(a) else ifnone(c, a)

def ifelse(a, b, c):
    "`b` if `a` is True else `c`"
    return b if a else c

# Cell
def stack(o, axis=0):
    if isinstance(o[0], np.ndarray): return np.stack(o, axis)
    elif isinstance(o[0], torch.Tensor): return torch.stack(tuple(o), dim=axis)
    assert False, f'cannot stack this data type {type(o[0])}'

# Cell
# This is a convenience function will use later proposed by Thomas Capelle @tcapelle to be able to easily benchmark performance
def cycle_dl(dl):
    for _ in iter(dl): pass

# Cell
device = 'cuda' if torch.cuda.is_available() else 'cpu'
defaults.device = device
cpus = defaults.cpus


# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/100_layers.ipynb (unless otherwise specified).

__all__ = ['noop', 'same_padding1d', 'ZeroPad1d', 'ConvSP1d', 'convlayer']

# Cell
import torch
import torch.nn as nn

def noop(x): return x

def same_padding1d(seq_len,ks,stride=1,dilation=1):
    assert stride > 0
    assert dilation >= 1
    effective_ks = (ks - 1) * dilation + 1
    out_dim = (seq_len + stride - 1) // stride
    p = max(0, (out_dim - 1) * stride + effective_ks - seq_len)
    padding_before = p // 2
    padding_after = p - padding_before
    return padding_before, padding_after

class ZeroPad1d(nn.ConstantPad1d):
    def __init__(self, padding):
        super().__init__(padding, 0.)

class ConvSP1d(nn.Module):
    "Conv1d padding='same'"
    def __init__(self,c_in,c_out,ks,stride=1,padding='same',dilation=1,bias=True):
        super().__init__()
        self.ks, self.stride, self.dilation = ks, stride, dilation
        self.conv = nn.Conv1d(c_in,c_out,ks,stride=stride,padding=0,dilation=dilation,bias=bias)
        self.zeropad = ZeroPad1d
        self.weight = self.conv.weight
        self.bias = self.conv.bias

    def forward(self, x):
        padding = same_padding1d(x.shape[-1],self.ks,stride=self.stride,dilation=self.dilation)
        return self.conv(self.zeropad(padding)(x))

def convlayer(c_in,c_out,ks=3,padding='same',bias=True,stride=1,
              bn_init=False,zero_bn=False,bn_before=True,act_fn=True,**kwargs):
    '''conv layer (padding="same") + bn + act'''
    if ks % 2 == 1 and padding == 'same': padding = ks // 2
    layers = [ConvSP1d(c_in,c_out, ks, bias=bias, stride=stride) if padding == 'same' else \
    nn.Conv1d(c_in,c_out, ks, stride=stride, padding=padding, bias=bias)]
    bn = nn.BatchNorm1d(c_out)
    if bn_init: nn.init.constant_(bn.weight, 0. if zero_bn else 1.)
    if bn_before: layers.append(bn)
    if act_fn: layers.append(nn.ReLU())
    if not bn_before: layers.append(bn)
    return nn.Sequential(*layers)


import fastai2
import fastcore
import torch
from fastai2.imports import *
from fastai2.data.all import *
from fastai2.torch_core import *
from fastai2.learner import *
from fastai2.metrics import *
from fastai2.callback.all import *
from fastai2.vision.data import *
from fastai2.interpret import *
import pprint
import psutil
import scipy as sp

PATH = Path(os.getcwd())
device = 'cuda' if torch.cuda.is_available() else 'cpu'
from IPython.display import display, HTML